from django.apps import apps
from django.core import signing
from django.http import JsonResponse, Http404
from django.shortcuts import render
from django.middleware.csrf import get_token
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt

from .models import Plan, Subscription
from .forms import SubscribeForm, CancelSubscriptionForm
from stripe_subscription import utils


def session(request):
    """
    Returns current user session.
    """

    if request.user and request.user.is_authenticated:
        qs = Plan.objects.list_published()

        return JsonResponse({
            "status": "ok",
            "session": {
                "csrf_token": get_token(request),
                "user": request.user.email,
                "plans": [
                    {'id': p.id, 'title': p.title, 'price': p.price}
                    for p in qs
                ]
            }
        })

    return JsonResponse({"status": "error"}, status=400)


@login_required
def subscribe(request):
    """
    Creates new subscription for current user with payment token and plan.
    """

    if request.method == "POST":
        form = SubscribeForm(request.POST)

        if form.is_valid():
            token = form.cleaned_data['stripe_token']
            plan = form.cleaned_data['plan']
            quiz = form.cleaned_data['quiz']

            x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')

            if x_forwarded_for:
                client_ip = x_forwarded_for.split(',')[0]
            else:
                client_ip = request.META.get('REMOTE_ADDR')

            sub = Subscription.create_from_token(
                    token,
                    plan,
                    request.user,
                    quiz,
                    client_ip)

            if sub:
                return JsonResponse({
                    "status": "ok",
                    "deep_link": sub.deep_link
                })
        else:
            return JsonResponse(
                    {"status": "error", "errors": form.errors},
                    status=400)

    return JsonResponse({"status": "error"}, status=400)


@csrf_exempt
def cancel_subscription(request):
    """
    Sends confirmation email to cancel subscription.
    User email address retrieved from stripe Subscription payment method.
    :param request: Request object
    """

    form = CancelSubscriptionForm(request.POST)

    if not form.is_valid():
        data = {
            "status": "error",
            "errors": form.errors
        }
        return JsonResponse(data, status=400)

    try:
        obj = Subscription.objects.get(subscription_id=form.cleaned_data['subscription_id'])
        is_ok = utils.cancel_subscription_confirm_email(request.get_host(), obj)

        if is_ok:
            return JsonResponse({"status": "ok"})

    except Subscription.DoesNotExist:
        raise Http404

    return JsonResponse({"status": "error", "error": "Something went wrong"}, status=400)


def cancel_subscription_confirm(request, token):
    signer = signing.Signer()

    try:
        data = signer.unsign_object(token)
    except signing.BadSignature:
        raise Http404

    try:
        obj = Subscription.objects.get(pk=data['id'])
    except Subscription.DoesNotExist:
        raise Http404

    cfg = apps.get_app_config('stripe_subscription')
    template = cfg.get_template(request.get_host(), 'cancel-success')
    context = {
        'is_ok': utils.cancel_subscription(obj) if obj.is_active else True,
        'host': request.get_host()
    }

    return render(request, template, context)



from configparser import ConfigParser
from pathlib import Path

from django.apps import AppConfig
from django.conf import settings
from django.core import mail


class StripeSubscriptionConfig(AppConfig):
    name = 'stripe_subscription'

    def ready(self):
        """
        Loads custom per domain SMTP configuration.
        SMTP config sample:
            stripe_subscription_smtp.ini

            [example.com]
            backend=django.core.mail.backends.smtp.EmailBackend
            host=smtp.example.com
            port=587
            username=someuser
            password=somepassword
            use_ssl=True

        Notification config sample:
            stripe_subscription_notificaions.ini

            [example.com]
            from_email=no-reply@example.com
            cancel_subscription_email_template=stripe_subscription/cancel-email.html
            cancel_subscription_success_template=stripe_subscription/cancel-success.html
        """

        # 1. Load SMTP settings
        smtp_config_path = settings.BASE_DIR / 'stripe_subscription_smtp.ini'

        if smtp_config_path.exists():
            self._smtp_config = ConfigParser()
            self._smtp_config.read(smtp_config_path)
        else:
            self._smtp_config = None

        # 2. Load notifications settings
        notification_config_path = settings.BASE_DIR / 'stripe_subscription_notificaions.ini'

        if notification_config_path.exists():
            self._notification_config = ConfigParser()
            self._notification_config.read(notification_config_path)
        else:
            self._notification_config = None

    def get_smtp_connection(self, host):
        """
        Returns SMTP connection for give host.
        Per host configuration store in settings.BASE_DIR / stripe_subscription_smtp.ini.
        If there is no configuration found for given host Django defaul STMP connection will be returned.

        :param host: Host name
        :returns: SMTP connection object
        """
        cfg = {}
        
        if self._smtp_config and host in self._smtp_config:
            cfg = dict(self._smtp_config[host])

        return mail.get_connection(**cfg)

    def get_from_email(self, host):
        """
        Returns notfications email for given host.
        If there is notification config for given host settings.DEFAULT_FROM_EMAIL
        will be returned.
        :return str:
        """
        cfg = {}
        
        if host in self._notification_config:
            cfg = dict(self._notification_config[host])

        return cfg.get('from_email', settings.DEFAULT_FROM_EMAIL)

    def get_template(self, host, category):
        """
        Returns notification email template for given host and category
        :param host: Host name
        :param category: Notification category (f.e. cancel-email, cancel-success)
        """
        cfg = {}
        
        if host in self._notification_config:
            cfg = dict(self._notification_config[host])

        return cfg.get(category, 'stripe_subscription/%s.html' % category)



from django.urls import path

from . import views


app_name = 'stripe-subscription'

urlpatterns = [
    path('session/', views.session),
    path('subscribe/', views.subscribe),
    path('cancel/', views.cancel_subscription),
    path('cancel/<str:token>/confirm/', views.cancel_subscription_confirm, name='cancel-confirm'),
]



import logging
from datetime import datetime, timedelta

from django.apps import apps
from django.conf import settings
from django.core import mail
from django.template.loader import render_to_string

import stripe
stripe.api_key = settings.STRIPE_SECRET_KEY


logger = logging.getLogger(__name__)


def get_product(plan):
    """
    Returns stripe product id for given plan.
    :param plan: billing.models.Plan
    """

    if not plan.product_id:
        product = stripe.Product.create(
                name=plan.title)
        plan.product_id = product.id

    return plan.product_id


def create_price_on_stripe(plan):
    """
    Creates stripe.Price object via SDK.
    :param plan: billing.models.Plan
    """

    price = stripe.Price.create(
        unit_amount=int(plan.price * 100),
        currency="usd",
        product=get_product(plan),
        recurring={"interval": plan.recurring_interval})

    plan.price_id = price.id


def update_price_on_stripe(plan):
    """
    Removes stripe.Price object via SDK.
    :param plan: billing.models.Plan
    """
    stripe.Price.modify(
        plan.price_id,
        active=plan.published)


def get_trial_end(days=3):
    """
    Returns trial end timestamp.
    :param days: Trial period
    :returns int: Trial end timestamp
    """
    trial_end = datetime.now() + timedelta(days=days)
    return int(trial_end.timestamp())


def cancel_subscription_confirm_email(host, subscription):
    """
    Sends cancel subscription email for given host and subscription.
    :param host: Host name
    :param subscription: stripe_subscription.models.Subscription
    :returns bool: True if message sent successfully
    """
    try:
        sub = stripe.Subscription.retrieve(subscription.subscription_id)
        pm = stripe.PaymentMethod.retrieve(sub.default_payment_method)
    except Exception as e:
        logger.error('Faield to retrieve subscription form stripe API:\n%s', e)
        return False

    cfg = apps.get_app_config('stripe_subscription')
    connection = cfg.get_smtp_connection(host)

    with connection:
        email_template = cfg.get_template(host, 'cancel-email')
        html = render_to_string(email_template, {
            'host': host,
            'object': subscription,
            'subscription': sub,
            'payment_method': pm,
            'token': subscription.get_token()
        })

        msg = mail.EmailMessage(
            'Cancel subscription',
            html,
            cfg.get_from_email(host),
            [pm.billing_details.email],
            connection=connection
        )
        msg.content_subtype = "html"

        try:
            msg.send()
        except Exception as e:
            logger.error('Faield to send email with error:\n%s', e)
            return False

        return True


def cancel_subscription(subscription):
    """
    Immediately cancels given subscription via Stripe API
    :param subscription: stripe_subscription.models.Subscription
    """

    try:
        stripe.Subscription.delete(subscription.subscription_id)
    except Exception as e:
        logger.error('Faield to send email with error:\n%s', e)
        return False

    subscription.is_active = False
    subscription.save()

    return True



# Generated by Django 3.2.4 on 2021-06-09 10:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('stripe_subscription', '0003_trial_days'),
    ]

    operations = [
        migrations.AlterField(
            model_name='plan',
            name='price',
            field=models.DecimalField(decimal_places=2, max_digits=10),
        ),
    ]



# Generated by Django 3.2.2 on 2021-08-25 18:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('stripe_subscription', '0004_alter_plan_price'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='client_ip',
            field=models.GenericIPAddressField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='plan',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from django.conf import settings
from django.contrib import admin

import stripe

from . import models


@admin.register(models.Plan)
class PlanAdmin(admin.ModelAdmin):
    list_display = ('title', 'price_id', 'product_id', 'price', 'published')
    list_filter = ('published',)

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = ['price_id', 'product_id']

        if obj and obj.id:
            readonly_fields += ['price', 'recurring_interval']

        return readonly_fields


@admin.register(models.Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    change_form_template = 'stripe_subscription/admin/change-form.html'

    readonly_fields = (
            'created_at', 'updated_at', 'user',
            'plan', 'customer_id', 'subscription_id', 'quiz')
    list_display = ('user', 'created_at', 'updated_at',
                    'customer_id', 'subscription_id', 'is_active')
    list_filter = ('is_active', 'created_at')

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (None, {
                "fields": ['is_active', 'created_at', 'updated_at']
            }),
            ('Stripe', {
                "fields": ('plan', 'customer_id', 'subscription_id')
            }),
        ]

        if request.user.is_superuser:
            fieldsets[0][1]['fields'].insert(0, 'user')
            fieldsets.append(('Quiz', {"fields": ('quiz',)}))

        return fieldsets

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}

        obj = self.get_object(request, object_id)

        stripe.api_key = settings.STRIPE_SECRET_KEY

        if obj:
            customer = stripe.Customer.retrieve(obj.customer_id)
            subscription = stripe.Subscription.retrieve(obj.subscription_id)

            if subscription.default_payment_method:
                payment_method = stripe.PaymentMethod.retrieve(
                        subscription.default_payment_method)
            else:
                payment_method = None

            extra_context.update({
                'request': request,
                'customer': customer,
                'subscription': subscription,
                'payment_method': payment_method
            })

        return super().change_view(request, object_id, form_url, extra_context)



from datetime import datetime, timezone
from django import template

register = template.Library()


@register.filter
def epoch_to_date(unix_epoch):
    """
    Returns datetime object for given unix epoch time.
    :param unix_epoch: int Unix epoch time
    :returns datetime.datetime:
    """

    date = datetime.fromtimestamp(unix_epoch, tz=timezone.utc)

    return date



#!/usr/bin/env python3

import os
import re
import sys
import shutil
from setuptools import setup, find_packages

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 0)

# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================

This version of Django Model Editor requires Python {}.{}, but you're trying
to install it on Python {}.{}.
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('stripe_subscription')


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    if os.system("twine check dist/*"):
        print("twine check failed. Packages might be outdated.")
        print("Try using `pip install -U twine wheel`.\nExiting.")
        sys.exit()
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree('django_stripe_subscription.egg-info')
    sys.exit()


setup(
    name='django-stripe-subscription',
    version=version,
    url='https://github.com/python33/django-stripe-subscription',
    license='BSD',
    description='Stripe Subscription API client',
    author='Petr Dovnar',
    author_email='petrdovnar@gmail.com',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=["django>=2.0", "stripe>=2.54.0"],
    python_requires=">=3.0",
    zip_safe=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    project_urls={
        'Source': 'https://github.com/python33/django-stripe-subscription',
    },
)
